/*
 * m4atags
 * -------
 * Copyright 2006 by David Gerber <dg@zapek.com>
 * All Rights Reserved
 *
 * $Id$
 */

#include "m4atags.h"

/* public */
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

/* private */
#include "args.h"
#include "hosttype.h"


struct arguments args;


unsigned int parse_args(int argc, char **argv)
{
	int c;
	char *name = argv[0];
	unsigned int showhelp = FALSE;

	/*
	 * Default values.
	 */
	
	args.flags = 0;

	while ((c = getopt(argc, argv, ARG_TEMPLATE)) != -1)
	{
		switch (c)
		{
			case 's': /* strip */
				args.flags |= ARG_STRIP;
				break;

			case 'v': /* version */
				printf(APPNAME "/" VERSION "\n");
				return (FALSE);

			case 'V': /* version and build date */
				printf(APPNAME "/" VERSION", " HOSTTYPE ", " __DATE__" " __TIME__"\n");
				return (FALSE);

			case 'y': /* year */
				{	
					int year = atoi(optarg);
					if (year < 1 || year > 10000)
					{
						fprintf(stderr, "invalid year argument, should be a number like 2006.\n");
						return (FALSE);
					}
					args.year = optarg;
				}
				break;

			case 'e': /* encoder */
				args.encoder = optarg;
				break;

			case 'a': /* artist */
				args.artist = optarg;
				break;

			case 'A': /* album */
				args.album = optarg;
				break;

			case 'n': /* name */
				args.name = optarg;
				break;

			case 'w': /* writer */
				args.writer = optarg;
				break;

			case 'c': /* comment */
				args.comment = optarg;
				break;

			case 'g': /* genre */
				args.genre = optarg;
				break;

			case 't': /* track */
				args.track = atoi(optarg);
				if (args.track < 1 || args.track > 8000)
				{
					fprintf(stderr, "invalid track argument, should be a number like 5.\n");
					return (FALSE);
				}
				break;

			case 'T': /* total tracks */
				args.total_tracks = atoi(optarg);
				if (args.total_tracks < 1 || args.total_tracks > 8000)
				{
					fprintf(stderr, "invalid total tracks argument, should be a number like 5.\n");
					return (FALSE);
				}
				break;

			case 'd': /* disk */
				args.disk = atoi(optarg);
				if (args.disk < 1 || args.disk > 8000)
				{
					fprintf(stderr, "invalid disk argument, should be a number like 5.\n");
					return (FALSE);
				}
				break;

			case 'D': /* total disks */
				args.total_disks = atoi(optarg);
				if (args.total_disks < 1 || args.total_disks > 8000)
				{
					fprintf(stderr, "invalid total disks argument, should be a number like 5.\n");
					return (FALSE);
				}
				break;

			case 'h': /* help */
				showhelp = TRUE;
				break;

			case ':': /* missing parameter */
				showhelp = TRUE;
				break;

			case '?': /* unknown option character */
				showhelp = TRUE;
				break;
		
			#ifdef DEBUG
			default:
				DB(("unknown getopt() return value\n"));
				return (FALSE);
			#endif
		}
	}

	argc -= optind;
	argv += optind;

	if (argc >= 1)
	{
		args.filename = argv[0];
	}

	if (!args.filename)
	{
		showhelp = TRUE;
	}

	if (showhelp)
	{
		fprintf(stderr, "usage: %s [options] [filename]\n"
						"options:\n"
						"-s            strip old tags\n"
						"-e <encoder>  encoder's name\n"
						"-a <artist>   artist's name\n"
						"-A <album>    album's name\n"
						"-n <name>     name of the song\n"
						"-w <writer>   composer of the song, usually empty\n"
						"-g <genre>    genre of the song (eg. \"Hard Trance\")\n"
						"-c <comment>  comment\n"
						"-t <track>    track number (eg. 5)\n"
						"-T <tracks>   total number of tracks in the album\n"
						"-d <disk>     disk number (eg. 8)\n"
						"-D <disks>    total number of disks in the collection\n"
						"-y <year>     year of song's creation (eg. 2006)\n"
						"-v            display version informations\n"
						"-V            display version and build informations\n"
						"\n"
						, name);
		return (FALSE);
	}
	return (TRUE);
}
